from collections import deque
import wx
from threading import Thread

from Downloader.download import Download
from Downloader.settings import Settings


class DownloadFrame(wx.Frame):

    def __init__(self, settings):
        self.settings = settings
        super().__init__(parent=None, size=self.settings.window_size, title=self.settings.window_title)
        self.Center()
        self.thread = deque(maxlen=5)
        self.panel = wx.Panel(parent=self)
        self.show_download_process_gauge = wx.Gauge(parent=self.panel,
                                                    style=wx.GA_HORIZONTAL | wx.GA_SMOOTH | wx.GA_TEXT,
                                                    size=wx.DefaultSize, pos=wx.DefaultPosition,
                                                    validator=wx.DefaultValidator,
                                                    name='download_process')
        self.show_download_files_text = wx.TextCtrl(parent=self.panel, style=wx.TE_MULTILINE | wx.TE_READONLY)
        self.file_button = wx.Button(parent=self.panel, label='file', id=3)
        self.search_button = wx.Button(parent=self.panel, label='Search', id=1)
        self.download_button = wx.Button(parent=self.panel, label='Download', id=2)
        self.search_text = wx.TextCtrl(parent=self.panel)
        self.choice_box = wx.Choice(parent=self.panel, choices=[])
        self.timer = wx.Timer(self)
        self.set_up()

    def set_up(self):
        vbox = wx.BoxSizer(wx.VERTICAL)
        hbox = wx.BoxSizer(wx.HORIZONTAL)
        hbox.Add(self.file_button, proportion=1, flag=wx.FIXED_MINSIZE | wx.CENTER, border=30)
        hbox.Add(self.search_text, proportion=2, flag=wx.FIXED_MINSIZE | wx.CENTER, border=30)
        hbox.Add(self.search_button, proportion=1, flag=wx.FIXED_MINSIZE | wx.CENTER, border=30)
        hbox.Add(self.choice_box, proportion=4, flag=wx.FIXED_MINSIZE | wx.CENTER, border=30)
        hbox.Add(self.download_button, proportion=1, flag=wx.FIXED_MINSIZE | wx.CENTER, border=30)
        vbox.Add(hbox, proportion=1, flag=wx.ALL | wx.EXPAND)
        download_info_box = wx.StaticBox(parent=self.panel, label='Download Info')
        hsbox = wx.StaticBoxSizer(download_info_box, wx.VERTICAL)
        hsbox.Add(self.show_download_process_gauge, proportion=1, flag=wx.ALL | wx.SHAPED, border=10)
        hsbox.Add(self.show_download_files_text, proportion=4, flag=wx.ALL | wx.EXPAND, border=30)
        vbox.Add(hsbox, proportion=4, flag=wx.CENTER | wx.EXPAND)
        self.panel.SetSizer(vbox)
        self.bind_event()

    def bind_event(self):
        self.Bind(wx.EVT_BUTTON, self.search_onclick, id=1)
        self.Bind(wx.EVT_BUTTON, self.download_onclick, id=2)
        self.Bind(wx.EVT_BUTTON, self.choose_directory, id=3)
        self.Bind(wx.EVT_TIMER, self.show_download_info, self.timer)

    def search_onclick(self, event):
        book = self.search_text.GetValue()
        if book:
            self.download = Download(book, self.settings)
            self.download.search_related_book()
            choices = ['《' + i.title + '》' + i.author for i in self.settings.choose_urls][0:5]
            self.choice_box.SetItems(choices)
        else:
            return

    def reset(self):
        self.settings.reset()
        self.search_text.SetValue('')
        self.choice_box.SetItems([])
        self.show_download_process_gauge.SetValue(0)
        self.download_button.Enable()
        self.show_download_files_text.SetValue('')
        self.thread.popleft()

    def download_onclick(self, event):
        select = self.choice_box.GetSelection()
        if select >= 0:
            if not self.settings.store_directory_path:
                self.message_box("choose a directory to store", "WARNING!",self.choose_directory)
            if self.settings.store_directory_path:
                self.download.get_article_urls(select)
                self.download.mkdir(self.settings.store_directory_path)
                self.show_download_process_gauge.SetRange(self.settings.sum_tasks)
                self.thread.append(Thread(target=self.download.download))
                self.thread[-1].start()
                event.GetEventObject().Disable()
                self.timer.Start(100)

    def show_download_info(self, event):
        if bool(self.settings.completed_article) or  self.settings.process<self.settings.sum_tasks:
            self._change_download_info()
        else:
            self.timer.Stop()
            self.message_box("the downloading task is completed,do you want to continue a new task?", "COMPLETED",
                             self.reset)

    def _change_download_info(self):
        self.show_download_process_gauge.SetValue(self.settings.process)
        if self.settings.completed_article:
            self.show_download_files_text.AppendText(self.settings.completed_article.pop() + '\n')

    def choose_directory(self, event=None):
        dialog = wx.DirDialog(None, "choose a directory:",
                              style=wx.DD_DEFAULT_STYLE | wx.DD_NEW_DIR_BUTTON)
        if dialog.ShowModal() == wx.ID_OK:
            self.settings.store_directory_path = dialog.GetPath()
        dialog.Destroy()

    def message_box(self,msg,title,handler):
        message_box = wx.MessageDialog(None, msg, title,
                                       wx.YES_NO | wx.ICON_QUESTION)
        if message_box.ShowModal() == wx.ID_YES:
            handler()
        message_box.Destroy()


class App(wx.App):
    def OnInit(self):
        settings = Settings()
        frame = DownloadFrame(settings)
        frame.Show()
        return True


def main():
    app = App()
    app.MainLoop()


if __name__ == '__main__':
    main()



import os

import gevent
from gevent import pool, monkey


monkey.patch_all(thread=False)

from Downloader.article import Article
from Downloader.book import Book


class Download:
    def __init__(self, book_name, settings):
        self.book = Book(book_name, settings)
        self.article = Article
        self.settings = settings
        self.directory_path = None

    def search_related_book(self):
        self.book.get_book_info()

    def get_article_urls(self, index):
        self.book.make_choice(index)
        self.book.get_urls_article()
        self.settings.sum_tasks = len(self.book.article_urls)

    def mkdir(self, directory_path):
        self.directory_path = os.path.join(directory_path, self.book.book_url[1])
        if os.path.exists(self.directory_path):
            return False
        else:
            os.mkdir(self.directory_path)
            return True

    def make_article(self, article_url):
        article = self.article(article_url)
        try:
            article.get_title()
            article.get_content()
            article.write(self.directory_path, self.settings.completed_article)
        except Exception:
            pass
        self.settings.process += 1

    def download(self):
        Pool = pool.Pool(self.settings.gevent_pool_num)
        gevent.joinall([Pool.spawn(self.make_article, i) for i in self.settings.article_urls])






import re
from collections import namedtuple
from bs4 import BeautifulSoup

from Downloader.url_request import UrlRequest


class Book:
    def __init__(self, search_book, settings):
        self.settings = settings
        self.search_url = settings.search_url
        self.article_urls = settings.article_urls
        self.book_name = search_book
        UrlRequest.pretend_header(self.settings.request_head_paras_file)
        self.url_request = UrlRequest('utf-8')
        self.book_url = None

    def get_book_info(self):
        data = {'keyword': self.book_name, 't': '1'}
        html = self.url_request.post(data, self.search_url)
        self._parse_book_html(html)

    def _parse_book_html(self, html):
        soup = BeautifulSoup(html, 'html.parser')
        books = soup.find_all('div', attrs={'class': 'hot_sale'})
        for book in books:
            self.settings.choose_urls.append(self._set_book(str(book)))

    @staticmethod
    def _set_book(html):
        Book = namedtuple('Book', ['id', 'title', 'author'])
        book_id = re.search(r'<a href="(/.*?/)">', html)
        title = re.search(r'<p class="title">(.*?)</p>', html)
        author = re.search(r'<p class="author">.*作者：(.*?)</p>', html)
        return Book(book_id[1], title[1], author[1])

    def make_choice(self,index):
        self.book_url = self.settings.choose_urls[index]

    def get_urls_article(self):
        if not isinstance(self.book_url, tuple):
            raise ValueError('Not choose a book')
        start_url = self.settings.index_url + self.book_url[0]
        html = self.url_request.get(start_url)
        soup = BeautifulSoup(html, 'html.parser')
        result = soup.find('div', attrs={'id': 'list'})
        pattern = r'<a href="%s(.*?)">' % (self.book_url[0])
        urls = re.findall(pattern, str(result))
        for i in range(len(urls)):
            self.article_urls.append(start_url + urls[i])



import requests


class UrlRequest:
    header_paras = {}

    def __init__(self, encoding=None):
        self.encoding = encoding

    @classmethod
    def pretend_header(cls, header_paras_file):
        with open(header_paras_file, 'r', encoding='utf-8') as f:
            for line in f:
                line = line.strip('\n').replace(' ', '')
                line_lst = line.split(':')
                if len(line_lst) == 2:
                    cls.header_paras[line_lst[0]] = line_lst[1]
                elif len(line_lst) > 2:
                    value = ''
                    for i in range(1, len(line_lst)):
                        value += line_lst[i]
                    cls.header_paras[line_lst[0]] = value
                else:
                    raise ValueError('File Not Have Right Syntax!')

    def get(self, url):
        try:
            req = requests.get(url, timeout=30, headers=self.header_paras)
            req.raise_for_status()
            req.encoding = req.apparent_encoding if not self.encoding else self.encoding
            return req.text
        except Exception as e:
            print(e)
            return False

    def post(self, data, url):
        if not isinstance(data, dict):
            raise ValueError('ValueError: dict type was expected')
        session = requests.Session()
        session_json = session.post(url=url, headers=self.header_paras, data=data)
        session_json.encoding = self.encoding if self.encoding else session_json.apparent_encoding
        return session_json.text



class Settings:
    def __init__(self):
        self.index_url = "https://www.biqooge.com"
        self.search_url = "https://m.biqooge.com/s.php"
        self.select_max = 5
        self.article_urls = []
        self.choose_urls = []
        self.completed_article = []
        self.threads_num = 5
        self.gevent_pool_num = 10
        self.request_head_paras_file = './sources/headers_content'
        self.window_size = (640, 480)
        self.window_title = 'NetworkLiteratureDownloader'
        self.process = 0
        self.sum_tasks = 0
        self.store_directory_path = None

    def reset(self):
        self.process = 0
        self.sum_tasks = 0
        self.article_urls = []
        self.choose_urls = []



import os
import re

from bs4 import BeautifulSoup

from Downloader.url_request import UrlRequest


class Article:
    def __init__(self,article_url):
        self.url_request = UrlRequest('gbk')
        self.html = self.url_request.get(article_url)
        self.title = None
        self.content = None

    def get_content(self):
        soup = BeautifulSoup(self.html,'html.parser')
        self.content = soup.find('div',attrs={'id':'content'}).text

    def get_title(self):
        raw_title = re.search(r'<h1>(.*?)</h1>', self.html)[1]
        self.format_title(raw_title)

    def format_title(self,raw_title):
        num_string = '零一二三四五六七八九十百千'
        result = re.search(r"第([%s]*?)章"%num_string, raw_title)
        if result:
            num_list = []
            for i in result.group(1):
                num_list.append(i)
            set_list = ['0', '0', '0', '0']
            if re.search('["十百千"]', result.group(1)):
                dict = {'百': 1, '千': 0, '十': 2}
                for i in range(len(num_list)):
                    if num_list[i] in dict.keys():
                        if i == 0:  # format the num range form ten to twenty
                            set_list[dict[num_list[i]]] = '1'
                        else:
                            set_list[dict[num_list[i]]] = str(num_string.index(num_list[i - 1]))
                if num_list[-1] not in dict.keys():  # format the num like '六百零一'
                    set_list[3] = str(num_string.index(num_list[-1]))
            else:
                # to format the title which doesn't have characters like '十百千'
                num_list.reverse()  # process the situation that the num doesn't have thousand position
                for i in range(len(num_list)):
                    set_list[3 - i] = str(num_string.index(num_list[i]))
            self.title = raw_title.replace(result[1],''.join(set_list))
        else:
            self.title = raw_title

    def write(self,filepath,completed_article,property='.txt'):
        path = os.path.join(filepath,self.title+property)
        with open(path,'w',encoding='utf-8')as f:
            f.write(self.content)
        completed_article.append(f'已下载完....{self.title}')




import os
from distutils.core import setup

this_directory = os.path.abspath(os.path.dirname(__file__))


def read_file(filename):
    with open(os.path.join(this_directory, filename))as f:
        long_description = f.read()
    return long_description


def read_requirements(filename):
    return [line.strip() for line in read_file(filename).splitlines()
            if not line.startswith('#')]


setup(
    name='NetworkLiteratureDownloader',
    python_requires='>=3.8.6',
    version='1.1.0',
    license='MIT',
    description='A Multi-Coroutine Downloader',
    long_description=read_file('README.md'),
    long_description_content_type="text/markdown",
    author='suyu',
    author_email='halfxianhuman@hotmail.com',
    packages=['Downloader'],
    url="https://https://test.pypi.org/legacy/NetworkLiteratureDownloader",
)
